Ddoc

$(SPEC_S Enums - Enumerated Types,

$(GRAMMAR
$(I EnumDeclaration):
	$(B enum) $(I Identifier) $(I EnumBody)
	$(B enum) $(I EnumBody)
	$(B enum) $(I Identifier) $(B :) $(I EnumBaseType) $(I EnumBody)
	$(B enum) $(B :) $(I EnumBaseType) $(I EnumBody)

$(I EnumBaseType):
	$(I Type)

$(I EnumBody):
	$(B ;)
	$(B {) $(I EnumMembers) $(B })

$(I EnumMembers):
	$(I EnumMember)
	$(I EnumMember) $(B ,)
	$(I EnumMember) $(B ,) $(I EnumMembers)

$(I EnumMember):
	$(I Identifier)
	$(I Identifier) $(B =) $(I AssignExpression)
)

	Enums are used to define a group of related integral constants.
	<p>

	If the enum $(I Identifier) is present, the $(I EnumMembers)
	are declared in the scope of the enum $(I Identifier).
	The enum $(I Identifier) declares a new type.
	<p>

	If the enum $(I Identifier) is not present, then the enum
	is an $(I anonymous enum), and the $(I EnumMembers) are declared
	in the scope the $(I EnumDeclaration) appears in.
	No new type is created; the $(I EnumMembers) have the type of the
	$(I EnumBaseType).
	<p>

	The $(I EnumBaseType) is the underlying type of the enum.
	It must be an integral type.
	If omitted, it defaults to $(B int).

------
enum { A, B, C }	// anonymous enum
------

	Defines the constants A=0, B=1, C=2 in a manner equivalent to:

------
const int A = 0;
const int B = 1;
const int C = 2;
------

	Whereas:

------
enum X { A, B, C }	// named enum
------

	Define a new type X which has values X.A=0, X.B=1, X.C=2
	<p>

	Named enum members can be implicitly cast to integral types, but integral types 
	cannot be implicitly cast to an enum type.
	<p>

	Enums must have at least one member.
	<p>

	If an $(I Expression) is supplied for an enum member, the value
	of the member is set to the result of the $(I Expression).
	The $(I Expression) must be resolvable at compile time.
	Subsequent enum members with no $(I Expression) are set to the
	value of the previous member plus one:

------
enum { A, B = 5+7, C, D = 8, E }
------

	Sets A=0, B=12, C=13, D=8, and E=9.

<h3>Enum Properties</h3>

<pre>
.init			First enum member value
.min			Smallest value of enum
.max			Largest value of enum
.sizeof			Size of storage for an enumerated value

For example:

X.min			is X.A
X.max			is X.C
X.sizeof		is same as int.sizeof
</pre>

<h3>Initialization of Enums</h3>

	In the absence of an explicit initializer, an enum variable
	is initialized to the first enum value.

------
enum X { A=3, B, C }
X x;		// x is initialized to 3
------

)

Macros:
	TITLE=D Programming Language - Enums
	WIKI=Enum

